#import the library used to send requests in Python as well as the library for encoding/decoding in base64
import requests
import base64

#defining global variables, in this case API_URL and API_KEY
API_URL = "https://api.price2spy.com/rest/v1/"
API_KEY = "XXXXXXXX"

#Defining a function for decoding the API_KEY
def decodeAPIKey(key):
    api_key_bytes = key.encode()
    encoded = base64.b64encode(api_key_bytes)
    return(encoded.decode('utf-8'))

#headers for the request, they determine the type and the API key needed
headers = {
    'accept': 'application/json',
    'Content-Type': 'application/json',
    'Authorization': 'Basic ' + decodeAPIKey(API_KEY),
}

#defining a function to get current pricing data from a category by passing the category ID
def getCurrentPricingData(categoryId):
    data = '{ "active": true, "categoryId": ' + str(categoryId) + '}'
    response = requests.post(API_URL + 'get-current-pricing-data', headers=headers, data=data)
    currentPricingDataResponse = response.json()
    return currentPricingDataResponse


#sends the request and stores the resonse in the variable "response" as serialized json
response = requests.get(API_URL + 'get-categories', headers=headers)

#deserializes the response so it becomes a dict 
#This part is wrapped in a try-catch block because it's the one that first raised an exception if the authentification didn't work as intended
#it can print whatever message it needs and be transparent as needed about it
try:
    currentPricingDataResponse = response.json()
except:
    print(response.content)


#loops through each category, prints it's name, calls the function for current pricing data for that category ID, loops through each product and 
#prints its name, all website URLs and prices on them. For this example, I wrote it to print the price and currency together but they can be separated if needed. 
#it's also in a try-catch block because it can raise an exception without it
try:
   for x in currentPricingDataResponse['category']:
        print("Category name: " + x.get('name') + "\n")
        currentPricingDataResponse = getCurrentPricingData(x.get('id'))
        for y in currentPricingDataResponse["products"]["product"]:
            print("Product name: " + y['productName'] + '\n')
            #print("Website URL: " + y['urls']['url'][0].get('url'))
            #print("Last measured price: " + str(y['urls']['url'][0]['lastMeasurement']['price']['amount']) + ' ' + y['urls']['url'][0]['lastMeasurement']['price']['currency'])
            for z in y['urls']['url']:
                print("Website URL: " + z.get('url'))
                print("Last measured price for that website: "  + str(z['lastMeasurement']['price'].get('amount')) + ' ' +z['lastMeasurement']['price'].get('currency')) 
                print('\n')
            print('\n')
except NameError as err:
    print("Name error: {0}".format(err))
except:
    print("Error occured.")

'''
Please note that Price2Spy changed the data types for ProductID, UrlID and AutomatchUrlID from int to long. 
That shouldn't be a problem in this example code as it uses the ProductID as a string and because newer versions of Python all support the increased 
int size (without having the "long" datatype anymore). However, if you're using older versions of Python and plan on using these fields as numbers,
 please change the "int" datatype to "long" in your code to avoid errors. 

'''
#---------------------------------------------------------------------------------------------------------------#

#Example response for the API_KEY: 54595f2eef22dc7d8f09c4de9373f81b34b99366 - remove this if necessary:

"""

Category name: Instrument Amplifiers

Product name: Arturia AudioFuse SpaceGray Audio-Interface

Website URL: https://www.bax-shop.nl/studio-monitor-controller/arturia-audiofuse-gray-audio-interface
Last measured price for that website: 0.0 EUR


Website URL: https://www.gear4music.de/de/Recording-and-Computer/Arturia-AudioFuse-USB-Interface-for-Mac-PC-and-iOS-Classic-Silver/16L1
Last measured price for that website: 0.0 EUR


Website URL: http://www.session.de/ARTURIA-AudioFuse-Space-Grey.html?listtype=search&searchparam=arturia
Last measured price for that website: 444.0 EUR


Website URL: https://www.justmusic.de/Recording/Interfaces/Audio-Interfaces/Arturia-AudioFuse-Space-Grey-Ausstellungsmodell
Last measured price for that website: 444.0 EUR


Website URL: http://www.musicworldbrilon.de/arturiaaudiofusespacegrey.html
Last measured price for that website: 469.0 EUR


Website URL: http://www.pourlesmusiciens.com/arturia-audiofuse-space-grey-interface-audio.html
Last measured price for that website: 543.0 EUR


Website URL: http://www.michenaud.com/p28356/arturia-audiofuse-g-space-grey.php
Last measured price for that website: 549.0 EUR


Website URL: http://www.keymusic.com/fr/produit/arturia-audiofuse-grey/?q=arturia&
Last measured price for that website: 599.0 EUR


Website URL: https://www.keymusic.com/nl/item/arturia-audiofuse-grey/?q=Arturia%20AudioFuse%20&
Last measured price for that website: 599.0 EUR




Product name: M-Cube GX_DE

Website URL: https://www.thomann.de/de/roland_micro_cube_gx.htm
Last measured price for that website: 135.0 EUR


Website URL: http://www.musicstore.de/de_DE/EUR/Roland-Micro-Cube-GX-Combo-BK-Black/art-GIT0028323-000
Last measured price for that website: 135.0 EUR


Website URL: https://www.woodbrass.com/de-de/e-gitarren-combos-modling-roland-micro-cube-gx-p164181.html
Last measured price for that website: 135.0 EUR


Website URL: http://www.musik-produktiv.de/roland-micro-cube-gx-black.html
Last measured price for that website: 135.0 EUR


Website URL: https://www.justmusic.de/de-de/gitarren-baesse/e-gitarren-verstaerker/combo/modelingverstaerker-combos/9128307/micro-cube-gx-bk.html
Last measured price for that website: 135.0 EUR


Website URL: http://www.musicworldbrilon.de/rolandmicrocubegxblack.html
Last measured price for that website: 135.0 EUR


Website URL: https://www.musikhaus-korn.de/de/Roland-Micro-Cube-GX-BK.html
Last measured price for that website: 135.0 EUR


Website URL: http://www.session.de/ROLAND-Micro-Cube-GX-BK.html?listtype=search&searchparam=mcube%20gx
Last measured price for that website: 135.0 EUR


Website URL: https://www.muziker.de/gitarren/gitarren-combos/combos-mit-transistor-verstarker/p/215340-roland-Micro-CUBE-GX-Black
Last measured price for that website: 135.0 EUR


Website URL: http://www.gear4music.de/de/Gitarre-and-Bass/Roland-Micro-Cube-GX-Gitarrenverstarker-schwarz/TXR
Last measured price for that website: 138.0 EUR


Website URL: https://www.bax-shop.fr/ampli-guitare-electrique/roland-micro-cube-gx-ampli-noir?q=9000-0005-8743
Last measured price for that website: 138.0 EUR


Website URL: https://www.bax-shop.de/roland-micro-cube-gx-schwarz
Last measured price for that website: 139.0 EUR


Website URL: http://www.klangfarbe.com/artikel.php?a=roland-cube-microcube-gx-bk-g41168
Last measured price for that website: 139.0 EUR


Website URL: https://www.thomann.de/es/roland_micro_cube_gx_bk.htm?ref=intl&shp=eyJjb3VudHJ5IjoiZXMiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6ImVzIn0%3D
Last measured price for that website: 139.0 EUR


Website URL: https://www.thomann.de/fr/roland_micro_cube_gx_bk.htm?ref=intl&shp=eyJjb3VudHJ5IjoiZnIiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6ImZyIn0%3D
Last measured price for that website: 139.0 EUR


Website URL: https://www.thomann.de/nl/roland_micro_cube_gx_bk.htm?ref=intl&shp=eyJjb3VudHJ5IjoibmwiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6Im5sIn0%3D
Last measured price for that website: 139.0 EUR


Website URL: https://www.musicstore.com/nl_NL/EUR/Roland-Micro-Cube-GX-combo-BK-zwart/art-GIT0028323-000
Last measured price for that website: 139.0 EUR


Website URL: https://www.keymusic.com/nl/item/roland-micro-cube-gx-guitar-amplifier/
Last measured price for that website: 139.0 EUR


Website URL: https://www.pourlesmusiciens.com/roland-micro-cube-gx-black-ampli-combo-guitare-electrique_72702.html
Last measured price for that website: 139.0 EUR


Website URL: https://www.bax-shop.nl/gitaarversterker-combo/roland-micro-cube-gx-zwart/product-details.html
Last measured price for that website: 141.0 EUR


Website URL: https://www.amazon.com/gp/offer-listing/B00JW286K8/ref=dp_olp_all_mbc?ie=UTF8&condition=all
Last measured price for that website: 159.14 USD




Product name: Roland CM30 Cube Monitor

Website URL: https://www.thomann.de/de/roland_cm_30_cubemonitor.htm
Last measured price for that website: 243.0 EUR


Website URL: https://www.muziker.de/studio-pa/studio-und-recording-equipment/studio-monitore/aktive-studio-monitore/p/207467-roland-CM-30-Cube-Monitor
Last measured price for that website: 243.0 EUR


Website URL: http://www.musik-produktiv.de/roland-cm-30.html
Last measured price for that website: 243.0 EUR


Website URL: https://www.justmusic.de/de-de/keys/keyboard-verstaerker/stereo-verstaerker/209946/cm-30.html
Last measured price for that website: 243.0 EUR


Website URL: https://www.woodbrass.com/de-de/tasteninstrumenten-zubehor-keyboard-verstarker-roland-cm-30-multi-purpose-monitor-p20465.html
Last measured price for that website: 243.0 EUR


Website URL: http://www.session.de/Roland-CM-30.html?listtype=search&searchparam=cm-30
Last measured price for that website: 243.0 EUR


Website URL: https://www.musicstore.de/de_DE/EUR/Roland-CM-30-Cube-Monitor/art-KEY0001744-000
Last measured price for that website: 243.0 EUR


Website URL: https://www.musikhaus-korn.de/en/roland-cm-30-cube-monitor/pd/8448?qsq=roland%20cm30
Last measured price for that website: 243.0 EUR


Website URL: https://www.bax-shop.de/roland-cm-30-cube-monitorspeaker
Last measured price for that website: 249.0 EUR


Website URL: https://www.thomann.de/fr/roland_cm_30_cubemonitor.htm?ref=intl&shp=eyJjb3VudHJ5IjoiZnIiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6ImZyIn0%3D
Last measured price for that website: 249.0 EUR


Website URL: https://www.thomann.de/es/roland_cm_30_cubemonitor.htm?ref=intl&shp=eyJjb3VudHJ5IjoiZXMiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6ImVzIn0%3D
Last measured price for that website: 249.0 EUR


Website URL: https://www.thomann.de/nl/roland_cm_30_cubemonitor.htm?ref=intl&shp=eyJjb3VudHJ5IjoibmwiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6Im5sIn0%3D
Last measured price for that website: 249.0 EUR


Website URL: https://www.musicstore.com/nl_NL/EUR/Roland-CM-30-Cube-Monitor-30-Watt-/art-KEY0001744-000;pgid=pnEgE04n_tZSRpd9HdGiSaER0000qgfd8_hr
Last measured price for that website: 249.0 EUR


Website URL: https://www.keymusic.com/nl/item/roland-cm-30-cube-monitor/?q=Roland%20CM30%20&
Last measured price for that website: 249.0 EUR


Website URL: https://www.bax-shop.fr/ampli-clavier/roland-cm-30-cube-moniteur-portatif?q=ROL-CM-30
Last measured price for that website: 251.0 EUR


Website URL: https://www.bax-shop.nl/actieve-fullrange/roland-cm-30-cube-monitorspeaker
Last measured price for that website: 251.0 EUR


Website URL: https://www.pourlesmusiciens.com/roland-cm30-amplificateur-sono_83451.html
Last measured price for that website: 254.0 EUR


Website URL: http://www.gear4music.de/de/PA-DJ-and-Lichttechnik/Roland-CM30-30W-CUBE-Monitor/7ZP
Last measured price for that website: 263.5 EUR


"""